
package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.BeanUtil;
import com.redxun.common.tool.IdGenerator;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.dao.KnowledgeCategory;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.mapper.KnowledgeCategoryMapper;
import com.redxun.knowledge.mapper.KnowledgeMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * [业务--知识分类表]业务服务类
 */
@Service
@Slf4j
public class KnowledgeCategoryServiceImpl extends SuperServiceImpl<KnowledgeCategoryMapper, KnowledgeCategory> implements BaseService<KnowledgeCategory> {

    @Resource
    private KnowledgeCategoryMapper knowledgeCategoryMapper;

    @Autowired
    private KnowledgeMapper knowledgeMapper;

    @Override
    public BaseDao<KnowledgeCategory> getRepository() {
        return knowledgeCategoryMapper;
    }

    // 根据id查询其下级分类信息(懒加载)
    public KnowledgeCategory findAllByCategoryId(String categoryId) {
        QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PK_ID", categoryId);
        KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(categoryId);
        Boolean hasChild = findHasChirld(knowledgeCategory);
        knowledgeCategory.setHasChild(hasChild);
        if (hasChild){
            List<KnowledgeCategory> allChildren = findAllChildren(knowledgeCategory);
            knowledgeCategory.setChildren(allChildren);
            allChildren.forEach(e -> {
                Boolean hasChirld = findHasChirld(e);
                e.setHasChild(hasChirld);
            });
        } else {
            knowledgeCategory.setChildren(findAllChildren(knowledgeCategory));
        }

        return knowledgeCategory;
    }

    /**
     * 知识分类列表(可带条件，条件为知识分类名称)
     * @param filter
     * @return
     */
    // @Cacheable(value = "knowledgeCategory",keyGenerator = "keyGenerator")
    public IPage findAllKnowledgeCategory(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        IPage page = knowledgeCategoryMapper.findAllKnowledgeCategory(filter.getPage(), filter.getParams(), params);
        List<KnowledgeCategoryAdminVo> knowledgeCategoryList = page.getRecords();
        // 查询顶级记录的所有子集合(顶级记录id为0)
        List<KnowledgeCategoryAdminVo> collect = knowledgeCategoryList.
                stream().
                filter(knowledgeCategoryAdminVo -> "0".equals(knowledgeCategoryAdminVo.getParent())).
                collect(Collectors.toList());
        // 查询其孩子节点
        collect.forEach(e -> {
            List<KnowledgeCategoryAdminVo> childrens = findChildrens(e);
            e.setChildren(childrens);
        });
        // 把数据放回到分页中，结果返回
        page.setRecords(collect);
        return page;
    }

    /**
     * 根据categoryId查询其父类信息(递归到祖先节点 1级分级)
     * @param categoryId
     * @return
     */
    public KnowledgeCategoryAdminVo findAllParentByCategoryId(String categoryId) {
        KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryMapper.selectByCategoryId(categoryId);
        // 如果父id等于0，则直接返回(0为顶级分类)
        if ("0".equals(knowledgeCategoryAdminVo.getParent())){
            return knowledgeCategoryAdminVo;
        }
        KnowledgeCategoryAdminVo allParent = findAllParent(knowledgeCategoryAdminVo.getParent());
        knowledgeCategoryAdminVo.setParents(allParent);
        return knowledgeCategoryAdminVo;
    }

    /**
     * 新增知识分类
     * @param entity
     * @return
     */
    @Override
    public int insert(KnowledgeCategory entity) {
        if (BeanUtil.isEmpty(entity.getPkId())) {
            entity.setPkId(IdGenerator.getIdStr());
        }
        Integer count = this.baseMapper.selectCount(new QueryWrapper<KnowledgeCategory>().eq("NAME", entity.getName()));
        if (count > 0){
            throw new RuntimeException("分类名称已存在,请重新输入");
        }
        // 没有parent默认为一级分类
        if (entity.getParent() == null){
            entity.setParent("0");
        }
        // 没有sort默认为0
        if (entity.getSort() == null){
            entity.setSort(0);
        }
        knowledgeCategoryMapper.insertOrUpdateBySort(entity.getParent(),entity.getSort());
        return this.getRepository().insert(entity);
    }

    /**
     * 修改知识分类
     * @param entity
     * @return
     */
    @Override
    public int update(KnowledgeCategory entity) {
        KnowledgeCategory knowledgeCategory = this.get(entity.getPkId());
        if (!knowledgeCategory.getName().equals(entity.getName())){
            Integer count = this.baseMapper.selectCount(new QueryWrapper<KnowledgeCategory>().eq("NAME", entity.getName()));
            if (count > 0){
                throw new RuntimeException("分类名称已存在,请重新输入");
            }
        }
        if (entity.getParent() != null){
            if (entity.getSort() == null) {
                entity.setSort(0);
            }
        }
        knowledgeCategoryMapper.insertOrUpdateBySort(entity.getParent(), entity.getSort());
        return this.getRepository().updateById(entity);
    }

    /**
     * 根据id进行数据回显
     * @param pkId
     * @return
     */
    public KnowledgeCategoryAdminVo findByCategoryId(String pkId) {
        KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryMapper.selectByCategoryId(pkId);
        KnowledgeCategoryAdminVo allParent = findAllParent(knowledgeCategoryAdminVo.getParent());
        knowledgeCategoryAdminVo.setParents(allParent);
        return knowledgeCategoryAdminVo;
    }

    /**
     * 删除知识分类(若分类下还有关联的知识，无法删除)
     * @param knowledgeCategoryId
     */
    @Transactional(rollbackFor = Exception.class)
    public Integer deleteKnowledgeCategory(String knowledgeCategoryId) {
        // 查询该分类下是否还有知识
        Boolean hasKnowledge = hasKnowledge(knowledgeCategoryId);
        if (hasKnowledge){
            return 0;
        }
        // 查询该分类的下是否还有子分类
        KnowledgeCategory knowledgeCategory = this.baseMapper.selectById(knowledgeCategoryId);
        Boolean hasChirld = findHasChirld(knowledgeCategory);
        if (hasChirld){
            return 1;
        }
        // 变更sort
        knowledgeCategoryMapper.deleteBySort(knowledgeCategory.getParent(),knowledgeCategory.getSort());
        knowledgeCategoryMapper.deleteKnowledgeCategory(knowledgeCategoryId);
        return 2;
    }



    // 查询孩子节点(非递归)
    private List<KnowledgeCategory> findAllChildren(KnowledgeCategory knowledgeCategory) {
        QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PARENT", knowledgeCategory.getPkId());
        List<KnowledgeCategory> knowledgeCategories = knowledgeCategoryMapper.selectList(queryWrapper);
        return knowledgeCategories;
    }

    // 查询孩子节点(递归)
    public List<KnowledgeCategoryAdminVo> findChildrens(KnowledgeCategoryAdminVo knowledgeCategoryAdminVo){
        List<KnowledgeCategoryAdminVo> knowledgeCategories = knowledgeCategoryMapper.selectKnowledgeCategoryByParent(knowledgeCategoryAdminVo.getPkId());
        knowledgeCategoryAdminVo.setChildren(knowledgeCategories);
        knowledgeCategories.forEach(e -> {
            List<KnowledgeCategoryAdminVo> allChildren = findChildrens(e);
            e.setChildren(allChildren);
        });
        return knowledgeCategories;
    }

    // 查询父亲节点(递归)
    private KnowledgeCategoryAdminVo findAllParent(String parentId){
        KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryMapper.selectByCategoryId(parentId);
        if (knowledgeCategoryAdminVo != null && !("0".equals(knowledgeCategoryAdminVo.getParent()))){
            KnowledgeCategoryAdminVo allParent = findAllParent(knowledgeCategoryAdminVo.getParent());
            knowledgeCategoryAdminVo.setParents(allParent);
        }
        return knowledgeCategoryAdminVo;
    }

    // 判断是否有孩子节点
    private Boolean findHasChirld(KnowledgeCategory knowledgeCategory) {
        QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PARENT", knowledgeCategory.getPkId());
        Integer count = knowledgeCategoryMapper.selectCount(queryWrapper);
        if (count > 0) {
            return true;
        } else {
            return false;
        }
    }

    // 判断分类下是否还有知识
    private Boolean hasKnowledge(String knowledgeCategoryId){
        QueryWrapper<Knowledge> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("CATEGORY_ID",knowledgeCategoryId);
        List<Knowledge> knowledges = knowledgeMapper.selectList(queryWrapper);
        return knowledges.size() > 0;
    }

    /**
     * 状态为true,上升;为false,下降
     * @param knowledgeCategoryId
     * @param status
     */
    @Transactional(rollbackFor = Exception.class)
    public void goUpOrDownUp(String knowledgeCategoryId, Boolean status) {
        KnowledgeCategory knowledgeCategory = this.get(knowledgeCategoryId);
        Integer sort = knowledgeCategory.getSort();
        String parent = knowledgeCategory.getParent();
        if (status){
            // 上升
            QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("PARENT",parent);
            queryWrapper.orderByAsc("SORT");
            List<KnowledgeCategory> knowledgeCategoryList = knowledgeCategoryMapper.selectList(queryWrapper);
            List<KnowledgeCategory> collect = knowledgeCategoryList.stream().filter(e -> e.getSort() < sort).collect(Collectors.toList());
            KnowledgeCategory knowledgeCategoryPre = collect.get(collect.size() - 1);
            // 交换位置
            int temp = sort;
            knowledgeCategoryMapper.swapSort(knowledgeCategoryId,knowledgeCategoryPre.getSort());
            knowledgeCategoryMapper.swapSort(knowledgeCategoryPre.getPkId(),temp);
        } else {
            // 下降
            QueryWrapper<KnowledgeCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("PARENT",parent);
            queryWrapper.orderByAsc("SORT");
            List<KnowledgeCategory> knowledgeCategoryList = knowledgeCategoryMapper.selectList(queryWrapper);
            List<KnowledgeCategory> collect = knowledgeCategoryList.stream().filter(e -> e.getSort() > sort).collect(Collectors.toList());
            KnowledgeCategory knowledgeCategoryNext = collect.get(0);
            // 交换位置
            int temp = sort;
            knowledgeCategoryMapper.swapSort(knowledgeCategoryId,knowledgeCategoryNext.getSort());
            knowledgeCategoryMapper.swapSort(knowledgeCategoryNext.getPkId(),temp);
        }
    }

    /**
     * PC端展示(无分页,数据直接返回)
     * @param
     * @return
     */
    public List<KnowledgeCategoryAdminVo> getAllKnowledgeCategory() {
        List<KnowledgeCategoryAdminVo> knowledgeCategoryList = knowledgeCategoryMapper.getAllKnowledgeCategory();
        // 查询顶级记录的所有子集合(顶级记录id为0)
        List<KnowledgeCategoryAdminVo> collect = knowledgeCategoryList.
                stream().
                filter(knowledgeCategoryAdminVo -> "0".equals(knowledgeCategoryAdminVo.getParent())).
                collect(Collectors.toList());
        // 查询其孩子节点
        collect.forEach(e -> {
            List<KnowledgeCategoryAdminVo> childrens = findChildrens(e);
            e.setChildren(childrens);
        });
        return collect;
    }
}
